

class File(object):

    def __init__(self):

        self.filename = ''
        self.server_filename = ''
        self.rotate = 0
        self.password = None
        # file_encryption_key TBD
        self.file_encryption_key = None
        self.metas = {}

    @property
    def metas_values(self):

        return "Title", "Author", "Subject", "Keywords", "Creator", "Producer", "CreationDate", "ModDate", "Trapped"

    def get_file_options(self):
        pass

    def set_metas(self, key, value):

        if key in self.metas_values:
            self.metas[key] = value
        else:
            raise ValueError("'%s' is not a meta tag: %s" % (key, self.metas))

    def as_dict(self):

        for k in dir(self):
            if getattr(self, k) and \
                    'set_metas' not in k and 'get_file_options' not in k and 'as_dict' not in k \
                    and '__' not in k and '_values' not in k:
                yield k, getattr(self, k)




from edenpdf.tools.validatepdfa import ValidatePdfA

t = ValidatePdfA('public_key', verify_ssl=True)
t.add_file('pdf_file')
t.debug = False
t.conformance = 'pdfa-2b'

t.execute()
t.delete_current_task()



from edenpdf.task import Task


class Watermark(Task):

    def __init__(self, public_key, verify_ssl, proxies):

        self.tool = 'watermark'

        self.vertical_position = 'middle'
        self.horizontal_position = 'center'
        self.font_family = 'Arial Unicode MS'
        self.font_style = None
        self.layer = 'above'

        self.mode = None
        self.text = ''
        self.image = ''
        self.pages = ''
        self.vertical_position_adjustment = 0
        self.horizontal_position_adjustment = 0
        self.mosaic = False
        self.rotation = 0
        self.font_size = 14
        self.font_color = '#000000'
        self.transparency = 100

        super(Watermark, self).__init__(public_key, True, verify_ssl, proxies)

    @property
    def allowed_properties(self):
        return ['vertical_position', 'horizontal_position', 'font_family', 'font_style', 'layer', 'mode', 'text',
                'image', 'pages', 'vertical_position_adjustment', 'horizontal_position_adjustment', 'mosaic',
                'rotation', 'font_size', 'font_color', 'transparency']

    @property
    def vertical_position_values(self):
        return 'bottom', 'top', 'middle'

    @property
    def horizontal_position_values(self):
        return 'left', 'center', 'right'

    @property
    def font_family_values(self):
        return 'Arial', 'Arial Unicode MS', 'Verdana', 'Courier', 'Times New Roman', \
               'Comic Sans MS', 'WenQuanYi Zen Hei', 'Lohit Marathi'

    @property
    def font_style_values(self):
        return None, 'Bold', 'Italic'

    @property
    def layer_values(self):
        return 'above', 'below'

    def process(self):

        payload = super(Watermark, self).process()
        payload = self.as_dict(payload, self.allowed_properties)

        return payload









from edenpdf.task import Task


class PdfToJpg(Task):

    def __init__(self, public_key, verify_ssl, proxies):

        self.pdfjpg_mode = 'pages'

        self.tool = 'pdfjpg'
        super(PdfToJpg, self).__init__(public_key, True, verify_ssl, proxies)

    @property
    def allowed_properties(self):
        return 'pdfjpg_mode',

    @property
    def pdfjpg_mode_values(self):
        return 'pages', 'extract'

    def process(self):

        payload = super(PdfToJpg, self).process()
        payload = self.as_dict(payload, self.allowed_properties)

        return payload









from .ilovepdf import ILovePdf
from .tools.compress import Compress
from .tools.imagetopdf import ImageToPdf
from .tools.merge import Merge
from .tools.officepdf import OfficeToPdf
from .tools.pagenumber import Pagenumber
from .tools.pdfa import ToPdfA
from .tools.pdftojpg import PdfToJpg
from .tools.protect import Protect
from .tools.rotate import Rotate
from .tools.split import Split
from .tools.unlock import Unlock
from .tools.validatepdfa import ValidatePdfA
from .tools.watermark import Watermark



import os
import re
import uuid

from edenpdf.file import File
from edenpdf.ilovepdf import ILovePdf


class Task(ILovePdf):

    tool = ''

    def __init__(self, public_key, start=False, verify_ssl=True, proxies=None):

        self.files = []
        self.downloaded_file_extension = ''
        self.downloaded_filename = ''
        self.download_path = ''
        self.task = ''
        self.status = ''
        self.proxies = proxies

        super(Task, self).__init__(public_key, proxies = self.proxies)

        self.begin = start
        self.file = None

        # Process task variables
        self.debug = False
        self.ignore_errors = True
        self.ignore_password = False
        # {date} = current date {n} = file number {filename} = original filename {tool} = current processing tool
        self.output_filename = '{filename}_{app}_{date}'
        # {date} = current date {n} = file number {filename} = original filename {app} = current processing action
        self.packaged_filename = 'archive_{app}_{date}'
        self.file_encryption_key = None
        self.try_pdf_repair = True
        self.custom_int = None
        self.custom_string = None
        self.webhook = None

        if self.begin:
            self._set_verify_ssl(verify_ssl)
            self.auth()
            self.start()

    def auth(self):

        payload = {"public_key": self.public_key}

        response = self._send_request('post', 'auth', payload, None, self.begin, proxies=self.proxies)
        self._set_token(response.token)
        self._set_headers()

    def start(self):

        headers = self.headers
        response = self._send_request('get', 'start/' + self.tool, None, headers, self.begin, proxies=self.proxies)
        self._set_working_server(response.server)

        self.task = response.task

    def add_file(self, file_path):

        self.file = File()

        self.file.filename = file_path
        self.files.append(self.file)

    def upload(self):

        payload = {"task": self.task}

        for file in self.files:

            with open(file.filename, 'rb') as f:
                response = self._send_request('post', 'upload', payload=payload, headers=self.headers,
                                              files={"file": f}, 
                                              proxies=self.proxies)

            file.server_filename = response.server_filename

    def add_file_from_url(self):
        pass

    def check_values(self, p, p_list_of_values):

        value = getattr(self, p)
        try:
            list_of_values = getattr(self, p_list_of_values)
        except AttributeError:
            # for example self.mode does not have self.mode_values
            return True

        if value in list_of_values:
            return True
        else:
            return False

    def as_dict(self, payload, allowed_properties):

        """Builds a dictionary payload

        It uses allowed properties from and object to build a payload dictionary.
        Payload is a command to be sent to get ilovepdf.com working
        It is called inside tools

        :param payload: commands
        :param allowed_properties:
        :type payload: dict
        :type allowed_properties: tuple, list
        :return: payload
        :rtype: dict
        """

        for k in allowed_properties:
            if self.check_values(k, k + '_values'):
                payload[k] = getattr(self, k)
            else:
                raise ValueError("'%s' is not allowed in '%s' property valid values: (%s)"
                                 % (getattr(self, k), k, getattr(self, k + '_values')))

        return payload

    def process(self):

        """Builds payload commands to get ilovepdf.com working

        :return: payload
        :rtype: dict
        """

        payload = {'task': self.task, 'tool': self.tool, 'ignore_errors': self.ignore_errors,
                   'ignore_password': self.ignore_password, 'output_filename': self.output_filename,
                   'packaged_filename': self.packaged_filename, 'try_pdf_repair': self.try_pdf_repair,
                   'custom_string': self.custom_string, 'webhook': self.webhook}

        if self.debug:
            payload['debug'] = True
            print("n of files: %s" % len(self.files))
            print(self.files)

        i = 0
        # adding commands for each file filtering some file object properties
        for file in self.files:
            for attribute, value in iter(file.as_dict()):
                if attribute != 'password' and attribute != 'file_encryption_key' and attribute != 'metas':
                    index = "files[%s][%s]" % (i, attribute)
                    payload[index] = value
                elif attribute == 'password':
                    if value:
                        payload[attribute] = value
                elif attribute == 'file_encryption_key':
                    if value:
                        payload['file_encryption_key'] = value
                elif attribute == 'metas':
                    for k, v in value.items():
                        if v:
                            index = "metas[%s]" % k
                            payload[index] = v
            i += 1

        if self.debug:
            print("\nBuilt Payload: %s" % payload)

        return payload

    def execute(self):

        print('Uploading file...')
        self.upload()

        payload = self.process()

        response = self._send_request('post', 'process', payload, self.headers, proxies=self.proxies)

        print("File uploaded! Below file stats:")
        self.status = response.status

        print(response)

    def set_output_folder(self, path):

        if not os.path.exists(path):
            os.makedirs(path)

        self.download_path = path

    def check_task_status(self, printall=False):

        response = self._send_request('get', 'task/%s' % self.task, None, self.headers, False,
                                      None, stream=False, proxies=self.proxies)

        if printall:
            print(response)
        else:
            return response.status

    def clean_filename(self, filename):
        """
        Clean filename from additional suffixes added by the server.
        Assures zip filename uniquess by adding a randomly generated string prefix.
        :param filename: filename to process
        """
        if ".zip" in filename:
            return str(uuid.uuid1())[:8]+ '_' + filename
        return "_".join(filename.split('_')[:-2]) + '.' + filename.rsplit(".", 1)[-1]

    def download(self):

        if len(self.files) > 0 and not self.debug and self.status == 'TaskSuccess':

            print('Downloading processed file...')

            response = self._send_request('get', 'download/%s' % self.task, None, self.headers, False,
                                          None, stream=True, proxies=self.proxies)

            # file_ext = str(response.headers['content-type']).split('/')

            filename = self.clean_filename(re.search(r'(filename=\")(.+\.\w+)(\")',
                                                     str(response.headers['content-disposition'])).group(2))

            with open(os.path.join(self.download_path, filename), 'wb') as f:
                for chunk in response.iter_content(10):
                    f.write(chunk)

            print('File downloaded!')

            return filename

        else:
            print("no file to be downloaded")

    def delete_current_task(self):

        response = self._send_request('post', 'task/' + self.task, None, headers=self.headers, proxies=self.proxies)
        print("Task delete %s" % response)

    def get_task_information(self):
        """
        Give information about a task status.
        If the task is TaskSuccess, TaskSuccessWithWarnings or TaskError it will also specify all files of the Task
        and their status one by one.
        :return: Response of the request.
        """
        return self._send_request('get', 'task/%s' % self.task, None, self.headers, False,
                                      None, stream=False, proxies=self.proxies)




import json


class Response(object):

    def __init__(self, response):

        self.status = response.status_code
        try:
            headers = response.headers['content-type']
            if 'application/pdf' in headers:
                setattr(self, 'headers', response.headers)
                setattr(self, 'iter_content', response.iter_content)
            elif 'application/zip' in headers:
                setattr(self, 'headers', response.headers)
                setattr(self, 'iter_content', response.iter_content)
            elif 'image/jpeg'in headers:
                setattr(self, 'headers', response.headers)
                setattr(self, 'iter_content', response.iter_content)
            else:
                raise KeyError
        except KeyError:
            try:
                params = json.JSONDecoder().decode(response.text)

                if params:
                    for attr, value in params.items():
                        setattr(self, attr, value)
                        
            except json.decoder.JSONDecodeError:
                pass

    def __str__(self):

        message = ''

        for element in dir(self):
            if '__' not in element and element:
                # in case we are validating pdfa we need more formatting controls
                if 'validations' in element:
                    value = getattr(self, element)
                    message += "%s: %s \n" % (element, value[0]['status'])
                # in case of errors we need more formatting
                elif 'error' in element:
                    message += 'Error '
                    for attr, val in getattr(self, element).items():
                        if 'code' not in attr:
                            if 'param' in attr:
                                if 'list' not in str(type(val)):
                                    for k, v in val.items():

                                        message += "%s: %s\n" % (k, v)
                                else:
                                    message += "Error: %s\n" % val[0]['error']
                            else:
                                message += "%s: %s\n" % (attr, val)

                else:
                    message += "%s: %s \n" % (element, getattr(self, element))

        return message



from edenpdf.ilovepdf import ILovePdf

ilovepdf = ILovePdf('public_key', verify_ssl=True)
task = ilovepdf.new_task('compress')
task.add_file('pdf_file')
task.debug = False
task.compression_level = 'low'
task.set_output_folder('output_directory')

task.execute()
task.download()
task.delete_current_task()



from edenpdf.tools.watermark import Watermark

t = Watermark('public_key', verify_ssl=True)
t.add_file('pdf_file')
t.debug = False
t.mode = 'text'
t.text = 'hi'
t.file.set_metas('Title', 'Hi, I am a title')
t.set_output_folder('output_directory')

t.execute()
t.download()
t.delete_current_task()



import requests
# lines below are to disable ssl warning when verify_ssl is set to False
import urllib3

from edenpdf.response import Response

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


class Request(object):

    @staticmethod
    def send(method, url, payload, headers=None, files=None, stream=None, verify_ssl=True, proxies=None):

        if method == 'post':
            response = requests.post(url, headers=headers, data=payload, files=files, verify=verify_ssl, stream=stream,
                                     proxies=proxies)
        else:
            response = requests.get(url, headers=headers, data=payload, files=files, verify=verify_ssl, stream=stream,
                                    proxies=proxies)

        return Response(response)



from edenpdf.tools.imagetopdf import ImageToPdf

t = ImageToPdf('public_key', verify_ssl=True)
t.add_file('pdf_file')
t.debug = False
t.orientation = 'portrait'
t.margin = 0
t.pagesize = 'fit'
t.set_output_folder('output_directory')

t.execute()
t.download()
t.delete_current_task()



from edenpdf.task import Task


class ValidatePdfA(Task):

    def __init__(self, public_key, verify_ssl, proxies):

        self.tool = 'validatepdfa'
        self.conformance = 'pdfa-2b'

        super(ValidatePdfA, self).__init__(public_key, True, verify_ssl, proxies)

    @property
    def allowed_properties(self):
        return 'conformance',

    @property
    def conformance_values(self):
        return 'pdfa-1b', 'pdfa-1a', 'pdfa-2b', 'pdfa-2u', 'pdfa-2a', 'pdfa-3b', 'pdfa-3u', 'pdfa-3a'

    def process(self):
        payload = super(ValidatePdfA, self).process()
        payload = self.as_dict(payload, self.allowed_properties)

        return payload








from edenpdf.task import Task


class Compress(Task):

    """Class to initialize the compress tool

    :param self.compression_level: the only param intended to be public

    """

    def __init__(self, public_key, verify_ssl, proxies):

        self.tool = 'compress'
        self.compression_level = 'recommended'

        super(Compress, self).__init__(public_key, True, verify_ssl, proxies)

    @property
    def allowed_properties(self):

        return 'compression_level',

    @property
    def compression_level_values(self):
        """allowed values for compression_level

        :return: tuple with available options
        """
        return 'low', 'extreme', 'recommended'

    def process(self):

        payload = super(Compress, self).process()
        payload = self.as_dict(payload, self.allowed_properties)

        return payload







# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['edenpdf', 'edenpdf.tools']

package_data = \
{'': ['*'], 'edenpdf': ['samples/*']}

install_requires = \
['requests>=2.22.0,<3.0.0', 'urllib3>=1.25.0,<2.0.0']

setup_kwargs = {
    'name': 'edenpdf',
    'version': '1.0.3',
    'description': 'ilovepdf.com python API library',
    'long_description': "# edenpdf\n\nilovepdf.com python API library, forked from Andrea Bruschi's pylovepdf\n\n## What it does\n\nThis library allows pdf manipulation using the API of http://www.ilovepdf.com. See the Tools section to know what you can do.\n\n### Prerequisites\n\n* [developer account] (https://developer.ilovepdf.com) to get a public key to use the API.\n* [Python >3.6]\n* [Requests] (http://it.python-requests.org/it/latest/)\n\n## Installation (older version)\n\n```\npip install pylovepdf \n```\n\n## Manual installation (up to date) \n\nDownload the latest release.\n```\npython setup.py install\n```\n\n## Getting started\n\nExample files are located inside samples/.\nChange file paths and the public_key parameter with the one you found in your developer account (see prerequisites).\nRun the files and enjoy.\n\n## Tools\nCurrently the following tools are available:\n\n* compress          (Reduce the size of pdf files)\n* imagepdf          (Converts an image to pdf)\n* merge             (Merge multiple pdf into single file)\n* officepdf         (Office document to pdf conversion)\n* pagenumber        (Place numbers on pages)\n* pdfa              (Converts into PDF/A)\n* pdfjpg            (Converts a pdf into jpeg image)\n* protect           (Add password to a pdf)\n* rotate            (Rotates the pages of a file)\n* split             (Split a pdf)\n* unlock            (Remove the password security from the pdf, the coolest feature ever!)\n* validatepdfa      (Checks the conformity of PDF/A format)\n* watermark         (Adds watermark to the file)\n\n## Example Usage (compress tool)\n```python\nfrom pylovepdf.ilovepdf import ILovePdf\n\nilovepdf = ILovePdf('public_key', verify_ssl=True)\ntask = ilovepdf.new_task('compress')\ntask.add_file('pdf_file')\ntask.set_output_folder('output_directory')\ntask.execute()\ntask.download()\ntask.delete_current_task()\n```\n\n## Alternative Example Usage (compress tool)\nA tool can be created directly:\n```python\nfrom pylovepdf.tools.compress import Compress\n\nt = Compress('public_key', verify_ssl=True)\nt.add_file('pdf_file')\nt.set_output_folder('output_directory')\nt.execute()\nt.download()\nt.delete_current_task()\n```\n## Documentation\n\nPlease see https://developer.ilovepdf.com/docs for up-to-date documentation.\n\n",
    'author': 'Eden-Box',
    'author_email': 'dev@edenbox.cf',
    'url': 'https://github.com/eden-box/edenpdf',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.0,<4.0.0',
}


setup(**setup_kwargs)



from edenpdf.task import Task


class Split(Task):

    def __init__(self, public_key, verify_ssl, proxies):

        self.tool = 'split'
        self.split_mode = 'ranges'
        self.ranges = None
        self.fixed_range = 1
        self.remove_pages = None
        self.merge_after = False

        super(Split, self).__init__(public_key, True, verify_ssl, proxies)

    @property
    def allowed_properties(self):

        allow = ['split_mode', 'ranges', 'fixed_range', 'remove_pages', 'merge_after']

        if self.split_mode == 'ranges':
            allow.remove('fixed_range')
        elif self.split_mode == 'fixed_range':
            allow.remove('ranges')

        if self.remove_pages == 'null':
            allow.remove('remove_pages')

        return allow

    @property
    def split_mode_values(self):
        return 'ranges', 'fixed_range', 'remove_pages'

    def process(self):
        payload = super(Split, self).process()
        payload = self.as_dict(payload, self.allowed_properties)

        return payload







from edenpdf.task import Task


class Unlock(Task):

    def __init__(self, public_key, verify_ssl, proxies):

        self.tool = 'unlock'
        super(Unlock, self).__init__(public_key, True, verify_ssl, proxies)






from edenpdf.tools.unlock import Unlock

t = Unlock('public_key', verify_ssl=True)
t.add_file('pdf_file')
t.debug = False
t.set_output_folder('output_directory')

t.execute()
t.download()
t.delete_current_task()



from edenpdf.tools.protect import Protect

t = Protect('public_key', verify_ssl=True)
t.add_file('pdf_file')
t.debug = False

# file_encryption key to be tested
t.file_encryption_key = 'ciao'

t.file.password = 'test'
t.set_output_folder('output_directory')

t.execute()
t.download()
t.delete_current_task()



from edenpdf.task import Task


class ImageToPdf(Task):

    def __init__(self, public_key, verify_ssl, proxies):

        self.orientation = 'portrait'
        self.margin = 0
        self.pagesize = 'fit'
        self.merge_after = True

        self.tool = 'imagepdf'
        super(ImageToPdf, self).__init__(public_key, True, verify_ssl, proxies)

    @property
    def allowed_properties(self):

        return 'orientation', 'margin', 'pagesize', 'merge_after'

    @property
    def pagesize_values(self):
        return 'fit', 'A4', 'letter'

    def process(self):

        payload = super(ImageToPdf, self).process()
        payload = self.as_dict(payload, self.allowed_properties)

        return payload









import importlib

from edenpdf.request import Request


class ILovePdf(object):
    """This class is used to init the API

        The public method you will use from this class is :func: `new_task`

        - **parameters**, **types**, **return** and **return types**::

              :param public_key: This is the API key to get on http://developers.ilovepdf.com
              :param verify_ssl: This parameter is used to enable or disable SSL during http requests
              :type public_key: String
              :type verify_ssl: Bool
              :return: return description
              :rtype: the return type description

        :Example:

            from edenpdf.ilovepdf import ILovePdf

            ilovepdf = ILovePdf('public_key', verify_ssl=True)
            task = ilovepdf.new_task('compress')
            task.add_file('pdf_file')
            task.set_output_folder('output_directory')
            task.execute()
            task.download()
            task.delete_current_task()

        Here below is the results of the :func:`new_task` docstring.

        """

    def __init__(self, public_key, verify_ssl=True, proxies=None):

        # auto explaining
        self.public_key = public_key
        self.ssl = verify_ssl
        self.proxies = proxies

        # Currently not used
        self.secret_key = ''

        self.api_version = 'v1'
        self.start_server = 'api.ilovepdf.com'
        self.working_server = ''

        # ilovepdf access token
        self.token = ''

        # headers which will contain access token to be sent for every task
        self.headers = None

    def _get_jwt(self):
        # TBD
        pass

    def _set_token(self, token):

        self.token = token

    def _set_headers(self):

        self.headers = {"Authorization": "Bearer " + self.token}

    def _set_working_server(self, working_server):

        self.working_server = working_server

    def _send_request(self, method, endpoint, payload, headers=None, start=False, files=None, stream=None, proxies=None):

        server = self.start_server

        if not start and self.working_server:
            server = self.working_server

        url = 'https://' + server + '/' + self.api_version + '/' + endpoint
        response = Request.send(method, url, payload, headers, files, stream, verify_ssl=self.ssl, proxies=proxies)

        return response

    def _set_verify_ssl(self, verify):

        self.ssl = verify

    def new_task(self, tool):
        """Returns Used tool Object

            This function instance a new object with the selected tool properties.

            - **parameters**, **types**, **return** and **return types**::

                  :param tool: name of the tool to be used
                  :type tool: String
                  :rtype: Tool Object

        """

        module_name = importlib.import_module('.tools.' + tool.lower(), package='pylovepdf')
        class_name = getattr(module_name, tool.title())

        return class_name(self.public_key, self.ssl, self.proxies)



from edenpdf.task import Task


class OfficeToPdf(Task):

    def __init__(self, public_key, verify_ssl, proxies):

        self.tool = 'officepdf'
        super(OfficeToPdf, self).__init__(public_key, True, verify_ssl, proxies)





